package grammar;

import parser.T;
import parser.T1;
import parser.Empty;
import parser.T3;

import java.util.ArrayList;

/**
 * BNF Sequence.
 *
 * @author Mathias Ricken
 */
public class Sequence implements Comparable {
    /**
     * List of symbols in this sequence.
     */
    private ArrayList<String> _symbols = new ArrayList<String>();

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        boolean first = true;
        for(String s: _symbols) {
            if (first) {
                first = false;
            }
            else {
                sb.append(" ");
            }
            sb.append(s);
        }
        return sb.toString();
    }

    /**
     * Create a sequence from a T.
     * @param t T
     */
    public Sequence(T t) {
        _symbols.add(t.getLexeme());
        t.getRest().execute(new T3.IT3Visitor<Object, ArrayList<String>>() {
            public Object tCase(T host, ArrayList<String> strings) {
                strings.add(host.getLexeme());
                return host.getRest().execute(this, strings);
            }
            public Object emptyCase(Empty host, ArrayList<String> strings) {
                return null;
            }
        }, _symbols);
    }

    /**
     * Create a sequence from a list of symbols.
     * @param symbols
     */
    public Sequence(ArrayList<String> symbols) {
        _symbols = new ArrayList<String>();
        for(String sym: symbols) {
            _symbols.add(new String(sym));
        }
    }

    /**
     * Copy constructor.
     *
     * @param s
     */
    public Sequence(Sequence s) {
        this(s._symbols);
    }

    /**
     * Accessor for the list of symbols.
     * @return list of symbols
     */
    public ArrayList<String> getSymbols() {
        return _symbols;
    }

    /**
     * Return true of this object and the other object are equal.
     * @param o other object
     * @return true if equal
     */
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Sequence)) return false;

        final Sequence sequence = (Sequence) o;

        if (_symbols != null ? !_symbols.equals(sequence._symbols) : sequence._symbols != null) return false;

        return true;
    }

    /**
     * Get hash code.
     * @return hash code
     */
    public int hashCode() {
        return (_symbols != null ? _symbols.hashCode() : 0);
    }

    /**
     * Compares this object with the specified object for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.

     * @param o the Object to be compared.
     * @return a negative integer, zero, or a positive integer as this object
     *         is less than, equal to, or greater than the specified object.
     * @throws ClassCastException if the specified object's type prevents it
     *                            from being compared to this Object.
     */
    public int compareTo(Object o) {
        if (this == o) return 0;
        if (!(o instanceof Sequence)) throw new ClassCastException();

        final Sequence sequence = (Sequence) o;

        return this.toString().compareTo(sequence.toString());
    }
}
