package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for ATVFactory.java.
 *
 * @author Mathias Ricken
 */
public class ATVFactoryGenerator {
    /**
     * Singleton instance.
     */
    public static final ATVFactoryGenerator Singleton = new ATVFactoryGenerator();

    /**
     * Private singleton constructor.
     */
    private ATVFactoryGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ATVFactory.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Abstract factory for a token visitor.");
        pw.println(" */");
        pw.println("public abstract class ATVFactory implements ITVFactory {");
        pw.println("    /**");
        pw.println("     * Tokenizer.");
        pw.println("     */");
        pw.println("    private ITokenizer _tokenizer;");
        pw.println();
        pw.println("    /**");
        pw.println("     * Constructor for the abstract factory.");
        pw.println("     *");
        pw.println("     * @param tkz tokenizer to use");
        pw.println("     */");
        pw.println("    public ATVFactory(ITokenizer tkz) {");
        pw.println("        _tokenizer = tkz;");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Return the next token.");
        pw.println("     *");
        pw.println("     * @return next token");
        pw.println("     */");
        pw.println("    protected final AToken nextToken() {");
        pw.println("        return _tokenizer.getNextToken();");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Put the last token back into the tokenizer.");
        pw.println("     */");
        pw.println("    protected final void putBackToken() {");
        pw.println("        _tokenizer.putBack();");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}
