package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for AToken.java.
 *
 * @author Mathias Ricken
 */
public class ATokenGenerator {
    /**
     * Singleton instance.
     */
    public static final ATokenGenerator Singleton = new ATokenGenerator();

    /**
     * Private singleton constructor.
     */
    private ATokenGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"AToken.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Abstract token class.");
        pw.println(" */");
        pw.println("public abstract class AToken {");
        pw.println("    /**");
        pw.println("     * The _lexeme this token is representing.");
        pw.println("     */");
        pw.println("    private String _lexeme;");
        pw.println();
        pw.println("    /**");
        pw.println("     * Token constructor.");
        pw.println("     *");
        pw.println("     * @param lexeme _lexeme this token is representing.");
        pw.println("     */");
        pw.println("    public AToken(String lexeme) {");
        pw.println("        _lexeme = lexeme;");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Execute a token visitor.");
        pw.println("     *");
        pw.println("     * @param algo  visitor to execute");
        pw.println("     * @param param visitor-specific parameter");
        pw.println("     * @return visitor-specific return value");
        pw.println("     */");
        pw.println("    public abstract Object execute(ITokVisitor algo, Object param);");
        pw.println();
        pw.println("    /**");
        pw.println("     * Return a string representation.");
        pw.println("     *");
        pw.println("     * @return string representation");
        pw.println("     */");
        pw.println("    public String toString() {");
        pw.println("        return _lexeme;");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}


