package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for EOFToken.java.
 * 
 * @author Mathias Ricken
 */
public class EOFTokenGenerator {
    /**
     * Singleton instance.
     */
    public static final EOFTokenGenerator Singleton = new EOFTokenGenerator();

    /**
     * Private singleton constructor.
     */
    private EOFTokenGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"EOFToken.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * End of file token class.");
        pw.println(" */");
        pw.println("public class EOFToken extends AToken {");
        pw.println("    /**");
        pw.println("     * Interface for a visitor to visit an eof token.");
        pw.println("     */");
        pw.println("    public static interface IVisitEOFToken extends ITokVisitor {");
        pw.println("        /**");
        pw.println("         * Case for an eof token.");
        pw.println("         *");
        pw.println("         * @param host  eof token");
        pw.println("         * @param param visitor-specific parameter");
        pw.println("         * @return visitor-specific return value");
        pw.println("         */");
        pw.println("        public Object eofCase(EOFToken host, Object param);");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Singleton instance.");
        pw.println("     */");
        pw.println("    public final static EOFToken Singleton = new EOFToken();");
        pw.println();
        pw.println("    /**");
        pw.println("     * Private singleton constructor.");
        pw.println("     */");
        pw.println("    private EOFToken() {");
        pw.println("        super(\"EOF\");");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Execute the eof case of the visitor.");
        pw.println("     *");
        pw.println("     * @param algo  visitor");
        pw.println("     * @param param visitor-specific parameter");
        pw.println("     * @return visitor-specific return value");
        pw.println("     */");
        pw.println("    public Object execute(ITokVisitor algo, Object param) {");
        pw.println("        return (algo instanceof IVisitEOFToken)");
        pw.println("                ? ((IVisitEOFToken) algo).eofCase(this, param)");
        pw.println("                : algo.defaultCase(this, param);");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}
