package grammar.util;

import grammar.Grammar;
import grammar.Definition;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.util.ArrayList;

/**
 * Generator for EmptyFact.java.
 *
 * @author Mathias Ricken
 */
public class EmptyFactGenerator {
    /**
     * Singleton instance.
     */
    public static final EmptyFactGenerator Singleton = new EmptyFactGenerator();

    /**
     * Private singleton constructor.
     */
    private EmptyFactGenerator() {
        // nothing to do
    }

    /**
     * Print with indent.
     **/
    private static void println(PrintWriter pw, int indent, String s) {
        for (int i=0; i<indent; ++i) {
            pw.print("    ");
        }
        pw.println(s);
    }

    public void generate(PrintWriter pw, int indent, String path, String packageName, Grammar g, ILambda accessSpecifier) throws FileNotFoundException {
        println(pw, indent, "/**");
        println(pw, indent, " * Factory for Empty grammar terminals.");
        println(pw, indent, " */");
        println(pw, indent, accessSpecifier.apply("Empty") + " class EmptyFact extends ATVFactory {");
        println(pw, indent, "    /**");
        println(pw, indent, "     * Constructor for Empty grammar terminals.");
        println(pw, indent, "     *");
        println(pw, indent, "     * @param tkz tokenizer to use");
        println(pw, indent, "     */");
        println(pw, indent, "    public EmptyFact(ITokenizer tkz) {");
        println(pw, indent, "        super(tkz);");
        println(pw, indent, "    }");
        println(pw, indent, "");
        println(pw, indent, "    /**");
        println(pw, indent, "     * Make a visitor to parse an empty terminal.");
        println(pw, indent, "     *");
        println(pw, indent, "     * @return visitor to parse an empty terminal");
        println(pw, indent, "     */");
        println(pw, indent, "    private ITokVisitor makeEmptyVis() {");
        println(pw, indent, "        return new ITokVisitor() {");
        println(pw, indent, "            public Object defaultCase(AToken host, Object param) {");
        println(pw, indent, "                putBackToken();");
        println(pw, indent, "                return Empty.Singleton;");
        println(pw, indent, "            }");
        println(pw, indent, "        };");
        println(pw, indent, "    }");
        println(pw, indent, "");
        println(pw, indent, "    /**");
        println(pw, indent, "     * Make a token visitor to parse an Empty terminal.");
        println(pw, indent, "     *");
        println(pw, indent, "     * @return token visitor");
        println(pw, indent, "     */");
        println(pw, indent, "    public ITokVisitor makeVisitor() {");
        println(pw, indent, "        return makeEmptyVis();");
        println(pw, indent, "    }");
        println(pw, indent, "");
        println(pw, indent, "    /**");
        println(pw, indent, "     * Make a token visitor that delegates to the given visitor in a chain of responsibility.");
        println(pw, indent, "     *");
        println(pw, indent, "     * @param successor visitor to serve as successor in the chain");
        println(pw, indent, "     */");
        println(pw, indent, "    public ITokVisitor makeChainedVisitor(ITokVisitor successor) {");
        println(pw, indent, "        return makeEmptyVis();");
        println(pw, indent, "    }");
    }
}
