package grammar.util;

import grammar.Grammar;
import grammar.Definition;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.util.ArrayList;

/**
 * Generator for Empty.java.
 *
 * @author Mathias Ricken
 */
public class EmptyGenerator {
    /**
     * Singleton instance.
     */
    public static final EmptyGenerator Singleton = new EmptyGenerator();

    /**
     * Private singleton constructor.
     */
    private EmptyGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"Empty.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Terminal: Empty");
        pw.println(" */");
        pw.print("public class Empty");

        ArrayList<Definition> referers = g.getRefererDefinitions("Empty");
        boolean first = true;

        for (Definition d: referers) {
            if (d.isAlternative()) {
                if (first) {
                    pw.println();
                    pw.print("    implements " + d.getName());
                    first = false;
                }
                else {
                    pw.println(",");
                    pw.print("               " + d.getName());
                }
            }
        }

        pw.println(" {");

        pw.println("    /**");
        pw.println("     * Singleton instance.");
        pw.println("     */");
        pw.println("    public final static Empty Singleton = new Empty();");
        pw.println();
        pw.println("    /**");
        pw.println("     * Private singleton constructor.");
        pw.println("     */");
        pw.println("    private Empty() {");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Return a string representation.");
        pw.println("     *");
        pw.println("     * @return string representation");
        pw.println("     */");
        pw.println("    public String toString() {");
        pw.println("        return \"\";");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}

