package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.util.ArrayList;

/**
 * Generator for factories.
 *
 * @author Mathias Ricken
 */
public class FilesFactoryGenerator {
    /**
     * Singleton instance.
     */
    public static final FilesFactoryGenerator Singleton = new FilesFactoryGenerator();

    /**
     * Private singleton constructor.
     */
    private FilesFactoryGenerator() {
        // nothing to do
    }

    /**
     * Generate the factory.
     * @param symbol symbol
     * @param path path for file
     * @param packageName package name
     * @param g grammar
     * @throws FileNotFoundException
     */
    public void generate(String symbol, String path, String packageName, Grammar g) throws FileNotFoundException {
        System.out.print("Generating factory for " + symbol);
        String name = symbol;

        if (symbol.startsWith("\"")) {
            name = quotedStringToName(g, symbol);
            System.out.print(" ("+name+")");
        }
        System.out.println();

        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+name+"Fact.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        if (symbol.equals("Empty")) {
            EmptyFactGenerator.Singleton.generate(pw, 0, path, packageName, g, new ILambda() {
                public Object apply(Object param) {
                    return "public";
                }
            });
        }
        else {
            FactoryGenerator.Singleton.generate(pw, 0, symbol, path, packageName, g, true, new ArrayList<String>(),  new ILambda() {
                public Object apply(Object param) {
                    return "public";
                }
            });
        }

        pw.println("}");

        pw.close();
    }

    /**
     * Get the name from a quoted string.
     * @param g grammar
     * @param symbol quoted string
     * @return name
     */
    private String quotedStringToName(Grammar g, String symbol) {
        String name;
        name = "_Terminal" + g.getTerminals().indexOf(symbol);
        return name;
    }
}
