package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for ITVFactory.java.
 *
 * @author Mathias Ricken
 */
public class ITVFactoryGenerator {
    /**
     * Singleton instance.
     */
    public static final ITVFactoryGenerator Singleton = new ITVFactoryGenerator();

    /**
     * Private singleton constructor.
     */
    private ITVFactoryGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ITVFactory.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Interface for an abstract factory for token visitors.");
        pw.println(" */");
        pw.println("public interface ITVFactory {");
        pw.println("    /**");
        pw.println("     * Make a token visitor.");
        pw.println("     *");
        pw.println("     * @return token visitor");
        pw.println("     */");
        pw.println("    public abstract ITokVisitor makeVisitor();");
        pw.println();
        pw.println("    /**");
        pw.println("     * Make a token visitor that delegates to the given visitor in a chain of responsibility.");
        pw.println("     *");
        pw.println("     * @param successor visitor to serve as successor in the chain");
        pw.println("     */");
        pw.println("    public abstract ITokVisitor makeChainedVisitor(ITokVisitor successor);");
        pw.println("}");

        pw.close();
    }
}
