package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for ITokVisitor.java.
 *
 * @author Mathias Ricken
 */
public class ITokVisitorGenerator {
    /**
     * Singleton instance.
     */
    public static final ITokVisitorGenerator Singleton = new ITokVisitorGenerator();

    /**
     * Private singleton constructor.
     */
    private ITokVisitorGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ITokVisitor.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Interface for a token visitor.");
        pw.println(" */");
        pw.println("public interface ITokVisitor {");
        pw.println("    /**");
        pw.println("     * Default case for a token.");
        pw.println("     *");
        pw.println("     * @param host  token");
        pw.println("     * @param param visitor-specific parameter");
        pw.println("     * @return visitor-specific return value");
        pw.println("     */");
        pw.println("    public Object defaultCase(AToken host, Object param);");
        pw.println("}");

        pw.close();
    }
}
