package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for ITokenizer.java.
 *
 * @author Mathias Ricken
 */
public class ITokenizerGenerator {
    /**
     * Singleton instance.
     */
    public static final ITokenizerGenerator Singleton = new ITokenizerGenerator();

    /**
     * Private singleton constructor.
     */
    private ITokenizerGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ITokenizer.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Extract and return an appropriate AToken from some given source.");
        pw.println(" */");
        pw.println("public abstract interface ITokenizer {");
        pw.println("    /**");
        pw.println("     * Return the next token.");
        pw.println("     */");
        pw.println("    public abstract AToken getNextToken();");
        pw.println("");
        pw.println("    /**");
        pw.println("     * Put the previously consumed token back into the token stream.");
        pw.println("     * Can only put back one token.");
        pw.println("     */");
        pw.println("    public abstract void putBack();");
        pw.println("}");

        pw.close();
    }
}
