package grammar.util;

import grammar.Grammar;
import grammar.Definition;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.util.ArrayList;

/**
 * Generator for ILambda.java and NoOpLambda.java.
 *
 * @author Mathias Ricken
 */
public class LambdaGenerator {
    /**
     * Singleton instance.
     */
    public static final LambdaGenerator Singleton = new LambdaGenerator();

    /**
     * Private singleton constructor.
     */
    private LambdaGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ILambda.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * Lambda interface.");
        pw.println(" */");
        pw.println("public interface ILambda {");
        pw.println("    /**");
        pw.println("     * Apply the lambda.");
        pw.println("     *");
        pw.println("     * @param param lambda-specific parameter");
        pw.println("     * @return lambda-specific return value");
        pw.println("     */");
        pw.println("    Object apply(Object param);");
        pw.println("}");

        pw.close();

        pw = new PrintWriter(new FileOutputStream(path+File.separator+"NoOpLambda.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("/**");
        pw.println(" * No-op lambda.");
        pw.println(" */");
        pw.println("public class NoOpLambda implements ILambda {");
        pw.println("    /**");
        pw.println("     * Singleton instance.");
        pw.println("     */");
        pw.println("    public static final NoOpLambda Singleton = new NoOpLambda();");
        pw.println();
        pw.println("    /**");
        pw.println("     * Private singleton constructor.");
        pw.println("     */");
        pw.println("    private NoOpLambda() {");
        pw.println("        // nothing to do");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Do nothing.");
        pw.println("     *");
        pw.println("     * @param nu not used");
        pw.println("     * @return null");
        pw.println("     */");
        pw.println("    public Object apply(Object nu) {");
        pw.println("        return null;");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}
