package grammar.util;

import grammar.Grammar;
import grammar.Definition;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.util.ArrayList;

/**
 * Generator for ParseEmptyFactory.java.
 *
 * @author Mathias Ricken
 */
public class ParseEmptyFactoryGenerator {
    /**
     * Singleton instance.
     */
    public static final ParseEmptyFactoryGenerator Singleton = new ParseEmptyFactoryGenerator();

    /**
     * Private singleton constructor.
     */
    private ParseEmptyFactoryGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"ParseEmptyFactory.java"), true);

        pw.println("package "+packageName+";");
        pw.println();
        pw.println("/**");
        pw.println(" * Factory for Empty grammar terminals.");
        pw.println(" */");
        pw.println("public class ParseEmptyFactory extends ATVFactory {");
        pw.println("    /**");
        pw.println("     * Constructor for Empty grammar terminals.");
        pw.println("     *");
        pw.println("     * @param tkz tokenizer to use");
        pw.println("     */");
        pw.println("    public ParseEmptyFactory(ITokenizer tkz) {");
        pw.println("        super(tkz);");
        pw.println("    }");
        pw.println("");
        pw.println("    /**");
        pw.println("     * Make a visitor to parse an empty terminal.");
        pw.println("     *");
        pw.println("     * @return visitor to parse an empty terminal");
        pw.println("     */");
        pw.println("    private IVisitDefault makeEmptyVis() {");
        pw.println("        return new IVisitDefault() {");
        pw.println("            public Object defaultCase(AToken host, Object param) {");
        pw.println("                putBackToken();");
        pw.println("                return Empty.Singleton;");
        pw.println("            }");
        pw.println("        };");
        pw.println("    }");
        pw.println("");
        pw.println("    /**");
        pw.println("     * Make a token visitor to parse an Empty terminal.");
        pw.println("     *");
        pw.println("     * @return token visitor");
        pw.println("     */");
        pw.println("    public IVisitDefault makeVisitor() {");
        pw.println("        return makeEmptyVis();");
        pw.println("    }");
        pw.println("");
        pw.println("    /**");
        pw.println("     * Make a token visitor to parse an Empty terminal that decorates the given visitor.");
        pw.println("     *");
        pw.println("     * @param decoree visitor to decorate");
        pw.println("     */");
        pw.println("    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {");
        pw.println("        return makeEmptyVis();");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}
