package grammar.util;

import grammar.Grammar;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;

/**
 * Generator for Tokenizer.java.
 * 
 * @author Mathias Ricken
 */
public class TokenizerGenerator {
    /**
     * Singleton instance.
     */
    public static final TokenizerGenerator Singleton = new TokenizerGenerator();

    /**
     * Private singleton constructor.
     */
    private TokenizerGenerator() {
        // nothing to do
    }

    public void generate(String path, String packageName, Grammar g) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(new FileOutputStream(path+File.separator+"Tokenizer.java"), true);

        if (!packageName.equals("")) {
            pw.println("package "+packageName+";");
            pw.println();
        }
        pw.println("import java.io.*;");
        pw.println();
        pw.println("/**");
        pw.println(" * Use the StreamTokenizer provided by in java.io to scan an input stream and extract an appropriate AToken.");
        pw.println(" */");
        pw.println("public class Tokenizer implements ITokenizer {");
        pw.println("    /**");
        pw.println("     * StreamTokenizer to use.");
        pw.println("     */");
        pw.println("    private StreamTokenizer _st;");
        pw.println();
        pw.println("    /**");
        pw.println("     * Reader to use.");
        pw.println("     */");
        pw.println("    private Reader _fileReader;");
        pw.println();
        pw.println("    /**");
        pw.println("     * Initialize _st to read from a input Reader file with the given input file name.");
        pw.println("     *");
        pw.println("     * @param inputFileName the name of the input text file");
        pw.println("     * @throws java.io.FileNotFoundException");
        pw.println("     */");
        pw.println("    public Tokenizer(String inputFileName) throws FileNotFoundException {");
        pw.println("        // Create an input text file object:");
        pw.println("        _fileReader = new FileReader(inputFileName);");
        pw.println();
        pw.println("        // Create a Streamtokenizer to parse the input text file:");
        pw.println("        _st = new StreamTokenizer(_fileReader);");
        pw.println();
        pw.println("        // TODO: set up StreamTokenizer");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Use _st to scan the input, extracts, and returns an appropriate concrete AToken.");
        pw.println("     *");
        pw.println("     * @return next token");
        pw.println("     * @throws IllegalArgumentException Thrown if an illegal input is encountered.");
        pw.println("     */");
        pw.println("    public AToken getNextToken() {");
        pw.println("        try {");
        pw.println("            if (StreamTokenizer.TT_EOF != _st.nextToken()) {");
        pw.println("                switch (_st.ttype) {");
        pw.println("                    // TODO: cases for Tokens");
        pw.println("                    // case '?':");
        pw.println("                    //    {");
        pw.println("                    //        return ...;");
        pw.println("                    //    }");
        pw.println("                    // case StreamTokenizer.TT_WORD:");
        pw.println("                    //     {");
        pw.println("                    //         return new WordToken(_st.sval);");
        pw.println("                    //     }");

        // TODO: Generate cases for quoted strings
//        boolean first = true;
//        for(String terminal: g.getTerminals()) {
//            if (terminal.startsWith("\"")) {
//                if (first) {
//                    pw.println();
//                    pw.println("                    // Cases for quoted strings");
//                    first = false;
//                }
//
//            }
//        }

        pw.println("                    default:");
        pw.println("                        throw new IllegalArgumentException(\"Illegal token (ttype=\" + new Character((char)_st.ttype) + \", sval=\" + _st.sval + \", nval=\" + _st.nval + \")!\");");
        pw.println("                }");
        pw.println("            }");
        pw.println("            else {");
        pw.println("                _fileReader.close();");
        pw.println("                return EOFToken.Singleton;");
        pw.println("            }");
        pw.println("        }");
        pw.println("        catch (IOException e) {");
        pw.println("            return null;");
        pw.println("        }");
        pw.println("    }");
        pw.println();
        pw.println("    /**");
        pw.println("     * Put the previously consumed token back into the token stream.");
        pw.println("     * Can only put back one token.");
        pw.println("     */");
        pw.println("    public void putBack() {");
        pw.println("        _st.pushBack();");
        pw.println("    }");
        pw.println("}");

        pw.close();
    }
}
