package parser;

/**
 * Assign token class.
 */
public class AssignToken extends AToken {
    /**
     * Interface for a visitor to visit a assign token.
     */
    public static interface IVisitAssignToken extends IVisitDefault {
        /**
         * Case for a assign token.
         *
         * @param host  assign token
         * @param param visitor-specific parameter
         * @return visitor-specific return value
         */
        public Object assignCase(AssignToken host, Object param);
    }

    /**
     * Singleton instance.
     */
    public final static AssignToken Singleton = new AssignToken();

    /**
     * Private singleton constructor.
     */
    private AssignToken() {
        super("::=");
    }

    /**
     * Execute the assign case of the visitor.
     *
     * @param algo  visitor
     * @param param visitor-specific parameter
     * @return visitor-specific return value
     */
    public Object execute(IVisitDefault algo, Object param) {
        return (algo instanceof IVisitAssignToken)
                ? ((IVisitAssignToken) algo).assignCase(this, param)
                : algo.defaultCase(this, param);
    }
}

