package parser;

/**
 * D non-terminal: D ::= word "::=" E D1
 *
 * @author Mathias Ricken
 */
public class D {
    /**
     * Terminal: word.
     */
    private WordToken _symbol;

    /**
     * Non-terminal: E.
     */
    private E _e;

    /**
     * Non-terminal: D1.
     */
    private D1 _d1;

    /**
     * Make a new D.
     * @param symbol symbol token
     * @param assign assign token
     * @param e E
     * @param d1 D1
     */
    public D(WordToken symbol, AssignToken assign, E e, D1 d1) {
        if (symbol.toString().equals("Empty")) {
            throw new IllegalArgumentException("Cannot define \"Empty\": reserved symbol");
        }
        _symbol = symbol;
        _e = e;
        _d1 = d1;
    }

    /**
     * Return a string representation.
     *
     * @return string representation
     */
    public String toString() {
        return _symbol.getLexeme() + " " + AssignToken.Singleton.getLexeme() + _e.toString() + _d1.toString();
    }

    /**
     * Get the symbol.
     * @return symbol
     */
    public WordToken getSymbol() {
        return _symbol;
    }

    /**
     * Get the expression
     * @return expression
     */
    public E getExpression() {
        return _e;
    }

    /**
     * Get the rest.
     * @return rest
     */
    public D1 getRest() {
        return _d1;
    }
}
