package parser;

/**
 * Or token class.
 */
public class OrToken extends AToken {
    /**
     * Interface for a visitor to visit an or token.
     */
    public static interface IVisitOrToken extends IVisitDefault {
        /**
         * Case for an or token.
         *
         * @param host  or token
         * @param param visitor-specific parameter
         * @return visitor-specific return value
         */
        public Object orCase(OrToken host, Object param);
    }

    /**
     * Singleton instance.
     */
    public final static OrToken Singleton = new OrToken();

    /**
     * Private singleton constructor.
     */
    private OrToken() {
        super("|");
    }

    /**
     * Execute the or case of the visitor.
     *
     * @param algo  visitor
     * @param param visitor-specific parameter
     * @return visitor-specific return value
     */
    public Object execute(IVisitDefault algo, Object param) {
        return (algo instanceof IVisitOrToken)
                ? ((IVisitOrToken) algo).orCase(this, param)
                : algo.defaultCase(this, param);
    }
}

