package parser;

/**
 * Factory for D1 grammar non-terminal objects.
 */
public class ParseD1Factory extends ATVFactory {
    /**
     * Factory for D1a grammar non-terminals.
     */
    private ParseD1aFactory _d1aFac;

    /**
     * Factory for Empty grammar terminals.
     */
    private ParseEmptyFactory _emptyFac;

    /**
     * Constructor for the D1 factory,
     *
     * @param tkz      tokenizer to use
     * @param d1aFac   factory for D1a non-terminals
     * @param emptyFac factory for empty terminals
     */
    public ParseD1Factory(ITokenizer tkz, ParseD1aFactory d1aFac, ParseEmptyFactory emptyFac) {
        super(tkz);
        _d1aFac = d1aFac;
        _emptyFac = emptyFac;

    }

    /**
     * Make a token visitor to parse an D1 non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return _d1aFac.makeDecVisitor(_emptyFac.makeVisitor());
    }

    /**
     * Make a token visitor to parse an D1 non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {
        return _d1aFac.makeDecVisitor(_emptyFac.makeDecVisitor(decoree));
    }
}

