package parser;

/**
 * Factory for D grammar non-terminal objects.
 */
public class ParseDFactory extends ATVFactory {
    /**
     * Token visitor to parse E grammar non-terminals.
     */
    private IVisitDefault _parseE;

    /**
     * Token visitor to parse D1 grammar non-terminals.
     */
    private IVisitDefault _parseD1;

    /**
     * Constructor for the D factory,
     *
     * @param tkz   tokenizer to use
     * @param eFac  factory for E non-terminals
     * @param d1Fac factory for D1 non-terminals
     */
    public ParseDFactory(ITokenizer tkz, ParseEFactory eFac, ParseD1Factory d1Fac) {
        super(tkz);
        _parseE = eFac.makeVisitor();
        _parseD1 = d1Fac.makeVisitor();
    }

    /**
     * Make an D grammar non-terminal.
     *
     * @param host some token
     * @param inp  visitor-specific input
     * @return D grammar non-terminal
     */
    private D makeD(AToken host, Object inp) {
        return new D((WordToken) host.execute(ParseWordVisitor.Singleton, inp),
                     (AssignToken) nextToken().execute(ParseAssignVisitor.Singleton, inp),
                     (E)nextToken().execute(_parseE, inp),
                     (D1)nextToken().execute(_parseD1, inp));
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return new IVisitDefault() {
            public Object defaultCase(AToken host, Object param) {
                return makeD(host, param);
            }
        };
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(final IVisitDefault decoree) {
        return new IVisitDefault() {
            public Object defaultCase(AToken host, Object param) {
                return makeD(host, param);
            }
        };
    }
}

