package parser;

/**
 * Factory for E1 grammar non-terminal objects.
 */
public class ParseE1Factory extends ATVFactory {
    /**
     * Factory for E1a grammar non-terminals.
     */
    private ParseE1aFactory _e1aFac;

    /**
     * Factory for Empty grammar terminals.
     */
    private ParseEmptyFactory _emptyFac;

    /**
     * Constructor for the E1 factory,
     *
     * @param tkz      tokenizer to use
     * @param e1aFac   factory for E1a non-terminals
     * @param emptyFac factory for empty terminals
     */
    public ParseE1Factory(ITokenizer tkz, ParseE1aFactory e1aFac, ParseEmptyFactory emptyFac) {
        super(tkz);
        _e1aFac = e1aFac;
        _emptyFac = emptyFac;

    }

    /**
     * Make a token visitor to parse an E1 non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return _e1aFac.makeDecVisitor(_emptyFac.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E1 non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {
        return _e1aFac.makeDecVisitor(_emptyFac.makeDecVisitor(decoree));
    }
}

