package parser;

/**
 * Factory for E grammar non-terminal objects.
 */
public class ParseEFactory extends ATVFactory {
    /**
     * Token visitor to parse T grammar non-terminals.
     */
    private IVisitDefault _parseT;

    /**
     * Token visitor to parse E1 grammar non-terminals.
     */
    private IVisitDefault _parseE1;

    /**
     * Constructor for the E factory,
     *
     * @param tkz   tokenizer to use
     * @param tFac  factory for T non-terminals
     * @param e1Fac factory for E1 non-terminals
     */
    public ParseEFactory(ITokenizer tkz, ParseTFactory tFac, ParseE1Factory e1Fac) {
        super(tkz);
        _parseT = tFac.makeVisitor();
        _parseE1 = e1Fac.makeVisitor();
    }

    /**
     * Make an E grammar non-terminal.
     *
     * @param host some token
     * @param inp  visitor-specific input
     * @return E grammar non-terminal
     */
    private E makeE(AToken host, Object inp) {
        return new E((T) host.execute(_parseT, inp),
                     (E1) nextToken().execute(_parseE1, inp));
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return new IVisitDefault() {
            public Object defaultCase(AToken host, Object param) {
                return makeE(host, param);
            }
        };
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(final IVisitDefault decoree) {
        return new IVisitDefault() {
            public Object defaultCase(AToken host, Object param) {
                return makeE(host, param);
            }
        };
    }
}

