package parser;

/**
 * Factory for Empty grammar terminals.
 */
public class ParseEmptyFactory extends ATVFactory {
    /**
     * Constructor for Empty grammar terminals.
     *
     * @param tkz tokenizer to use
     */
    public ParseEmptyFactory(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Make a visitor to parse an empty terminal.
     *
     * @return visitor to parse an empty terminal
     */
    private IVisitDefault makeEmptyVis() {
        return new IVisitDefault() {
            public Object defaultCase(AToken host, Object param) {
                putBackToken();
                return Empty.Singleton;
            }
        };
    }

    /**
     * Make a token visitor to parse an Empty terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return makeEmptyVis();
    }

    /**
     * Make a token visitor to parse an Empty terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {
        return makeEmptyVis();
    }
}

