package parser;

/**
 * Factory for QuotedString grammar terminals.
 */
public class ParseQuotedStringFactory extends ATVFactory {
    /**
     * Constructor for QuotedString grammar terminals.
     *
     * @param tkz tokenizer to use
     */
    public ParseQuotedStringFactory(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Make a token visitor to parse an Empty terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return new QuotedStringToken.IVisitQuotedStringToken() {
            public Object quotedStringCase(QuotedStringToken host, Object inp) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                throw new IllegalArgumentException("Wrong token: '" + host + "'");
            }
        };
    }

    /**
     * Make a token visitor to parse an D1a non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(final IVisitDefault decoree) {
        return new QuotedStringToken.IVisitQuotedStringToken() {
            public Object quotedStringCase(QuotedStringToken host, Object inp) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                return host.execute(decoree, param);
            }
        };
    }
}

