package parser;

/**
 * Factory for T3 grammar non-terminal objects.
 */
public class ParseT3Factory extends ATVFactory {
    /**
     * Factory for T grammar non-terminals.
     */
    private ParseTFactory _tFac;

    /**
     * Factory for Empty grammar terminals.
     */
    private ParseEmptyFactory _emptyFac;

    /**
     * Constructor for the T3 factory,
     *
     * @param tkz      tokenizer to use
     * @param tFac     factory for T non-terminals
     * @param emptyFac factory for Empty terminals
     */
    public ParseT3Factory(ITokenizer tkz, ParseTFactory tFac, ParseEmptyFactory emptyFac) {
        super(tkz);
        _tFac = tFac;
        _emptyFac = emptyFac;

    }

    /**
     * Constructor for the T3 factory,
     *
     * @param tkz      tokenizer to use
     * @param emptyFac factory for Empty terminals
     */
    public ParseT3Factory(ITokenizer tkz, ParseEmptyFactory emptyFac) {
        super(tkz);
        _emptyFac = emptyFac;

    }

    /**
     * Set the T factory,
     *
     * @param tFac     factory for T non-terminals
     */
    public void setParseTFactory(ParseTFactory tFac) {
        _tFac = tFac;
    }

    /**
     * Make a token visitor to parse an T non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return _tFac.makeDecVisitor(_emptyFac.makeVisitor());
    }

    /**
     * Make a token visitor to parse an T non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {
        return _tFac.makeDecVisitor(_emptyFac.makeDecVisitor(decoree));
    }
}
