package parser;

/**
 * Factory for T grammar non-terminal objects.
 */
public class ParseTFactory extends ATVFactory {
    /**
     * Factory for T1 grammar non-terminals.
     */
    private ParseT1Factory _t1Fac;

    /**
     * Factory for T2 grammar non-terminals.
     */
    private ParseT2Factory _t2Fac;

    /**
     * Constructor for the T factory,
     *
     * @param tkz      tokenizer to use
     * @param t1Fac    factory for T1 non-terminals
     * @param t2Fac    factory for T2 non-terminals
     */
    public ParseTFactory(ITokenizer tkz, ParseT1Factory t1Fac, ParseT2Factory t2Fac) {
        super(tkz);
        _t1Fac = t1Fac;
        _t2Fac = t2Fac;

    }

    /**
     * Make a token visitor to parse an T non-terminal.
     *
     * @return token visitor
     */
    public IVisitDefault makeVisitor() {
        return _t1Fac.makeDecVisitor(_t2Fac.makeVisitor());
    }

    /**
     * Make a token visitor to parse an T non-terminal that decorates the given visitor.
     *
     * @param decoree visitor to decorate
     */
    public IVisitDefault makeDecVisitor(IVisitDefault decoree) {
        return _t1Fac.makeDecVisitor(_t2Fac.makeDecVisitor(decoree));
    }
}
