package parser;

/**
 * Semicolon token class.
 */
public class SemicolonToken extends AToken {
    /**
     * Interface for a visitor to visit a semicolon token.
     */
    public static interface IVisitSemicolonToken extends IVisitDefault {
        /**
         * Case for a semicolon token.
         *
         * @param host  semicolon token
         * @param param visitor-specific parameter
         * @return visitor-specific return value
         */
        public Object semicolonCase(SemicolonToken host, Object param);
    }

    /**
     * Singleton instance.
     */
    public final static SemicolonToken Singleton = new SemicolonToken();

    /**
     * Private singleton constructor.
     */
    private SemicolonToken() {
        super(";");
    }

    /**
     * Execute the semicolon case of the visitor.
     *
     * @param algo  visitor
     * @param param visitor-specific parameter
     * @return visitor-specific return value
     */
    public Object execute(IVisitDefault algo, Object param) {
        return (algo instanceof IVisitSemicolonToken)
                ? ((IVisitSemicolonToken) algo).semicolonCase(this, param)
                : algo.defaultCase(this, param);
    }
}

