package parser;

/**
 * Non-terminal: T1 ::= word T3
 *
 * @author Mathias Ricken
 */
public class T1 implements T {
    /**
     * Terminal: word.
     */
    private WordToken _word;

    /**
     * Non-terminal: T3.
     */
    private T3 _t3;

    /**
     * Make a new T1.
     * @param word word
     * @param t3 T3
     */
    public T1(WordToken word, T3 t3) {
        _word = word;
        _t3 = t3;
    }

    /**
     * Return a string representation.
     *
     * @return string representation
     */
    public String toString() {
        return " " + _word.toString() + _t3.toString();
    }

    /**
     * Return the symbol.
     * @return symbol
     */
    public String getLexeme() {
        return _word.getLexeme();
    }

    /**
     * Return the rest.
     * @return rest
     */
    public T3 getRest() {
        return _t3;
    }

    /**
     * Visitor hook.
     *
     * @param v     visitor
     * @param param parameter
     * @return return value
     */
    public <R, P> R execute(T3.IT3Visitor<R, P> v, P param) {
        return v.tCase(this, param);
    }
}
