package parser;

/**
 * Non-terminal: T2 ::= word T3
 *
 * @author Mathias Ricken
 */
public class T2 implements T {
    /**
     * Terminal: quotedstring.
     */
    private QuotedStringToken _qs;

    /**
     * Non-terminal: T3.
     */
    private T3 _t3;

    /**
     * Make a new T3.
     * @param qs quotedstring
     * @param t3 T3
     */
    public T2(QuotedStringToken qs, T3 t3) {
        _qs = qs;
        _t3 = t3;
    }

    /**
     * Return a string representation.
     *
     * @return string representation
     */
    public String toString() {
        return " " + _qs.toString() + _t3.toString();
    }

    /**
     * Return the symbol.
     * @return symbol
     */
    public String getLexeme() {
        return _qs.getLexeme();
    }

    /**
     * Return the rest.
     * @return rest
     */
    public T3 getRest() {
        return _t3;
    }

    /**
     * Visitor hook.
     *
     * @param v     visitor
     * @param param parameter
     * @return return value
     */
    public <R, P> R execute(T3.IT3Visitor<R, P> v, P param) {
        return v.tCase(this, param);
    }
}
