package parser;

/**
 * Word token class.
 */
public class WordToken extends AToken {
    /**
     * Interface for a visitor to visit a word token.
     */
    public static interface IVisitWordToken extends IVisitDefault {
        /**
         * Case for a word token.
         *
         * @param host  word token
         * @param param visitor-specific parameter
         * @return visitor-specific return value
         */
        public Object wordCase(WordToken host, Object param);
    }

    /**
     * Constructor for a word token.
     *
     * @param lexeme word lexeme
     */
    public WordToken(String lexeme) {
        super(lexeme);
    }

    /**
     * Execute the word case of the visitor.
     *
     * @param algo  visitor
     * @param param visitor-specific parameter
     * @return visitor-specific return value
     */
    public Object execute(IVisitDefault algo, Object param) {
        return (algo instanceof IVisitWordToken)
                ? ((IVisitWordToken) algo).wordCase(this, param)
                : algo.defaultCase(this, param);
    }

    /**
     * Return a string representation.
     *
     * @return string representation
     */
    public String toString() {
        return getLexeme();
    }
}

