import java.io.*;
import output.*;

/**
 * Use the StreamTokenizer provided by in java.io to scan an input stream and extract an appropriate AToken.
 */
public class Tokenizer2 implements output.ITokenizer {
    /**
     * StreamTokenizer to use.
     */
    private StreamTokenizer _st;

    /**
     * Reader to use.
     */
    private Reader _fileReader;

    /**
     * Initialize _st to read from a input Reader file with the given input file name.
     *
     * @param inputFileName the name of the input text file
     * @throws java.io.FileNotFoundException
     */
    public Tokenizer2(String inputFileName) throws FileNotFoundException {
        // Create an input text file object:
        _fileReader = new FileReader(inputFileName);

        // Create a Streamtokenizer to parse the input text file:
        _st = new StreamTokenizer(_fileReader);

        _st.resetSyntax();
        _st.eolIsSignificant(false);
        _st.slashSlashComments(false);
        _st.slashStarComments(false);
        _st.quoteChar('"');
        _st.wordChars('a', 'z');
        _st.wordChars('A', 'Z');
        _st.wordChars('0', '9');
        _st.wordChars('_','_');
        _st.whitespaceChars(0,32);
    }

    /**
     * Use _st to scan the input, extracts, and returns an appropriate concrete AToken.
     *
     * @return next token
     * @throws IllegalArgumentException Thrown if an illegal input is encountered.
     */
    public AToken getNextToken() {
        try {
            if (StreamTokenizer.TT_EOF != _st.nextToken()) {
                switch (_st.ttype) {
                    case '|':
                        {
                            return _Terminal5.Singleton;
                        }
                    case ';':
                        {
                            return _Terminal4.Singleton;
                        }
                    case '"':
                        {
                            return new QuotedStringToken("\""+_st.sval+"\"");
                        }
                    case ':':
                        {
                            if (_st.nextToken() != ':') {
                                throw new IllegalArgumentException("Illegal token!");
                            }
                            if (_st.nextToken() != '=') {
                                throw new IllegalArgumentException("Illegal token!");
                            }
                            return _Terminal1.Singleton;
                        }
                    case StreamTokenizer.TT_WORD:
                        {
                            if (_st.sval.startsWith("_")) {
                                throw new IllegalArgumentException("Cannot define symbols starting with '_': reserved character");
                            }
                            if ((_st.sval.charAt(0) >= '0') && (_st.sval.charAt(0) <= '9')) {
                                throw new IllegalArgumentException("Cannot define symbols starting with '0' .. '9': reserved characters");                                
                            }
                            return new WordToken("" + _st.sval);
                        }
                    default:
                        throw new IllegalArgumentException("Illegal token (ttype=" + new Character((char)_st.ttype) + ", sval=" + _st.sval + ", nval=" + _st.nval + ")!");
                }
            }
            else {
                _fileReader.close();
                return EOFToken.Singleton;
            }
        }
        catch (IOException e) {
//            System.err.println(e.getMessage());
            return null;
        }
    }

    /**
     * Put the previously consumed token back into the token stream.
     * Can only put back one token.
     */
    public void putBack() {
        _st.pushBack();
    }
}
