package output;

/**
 * Factory for D1.
 */
public class D1Fact extends ATVFactory {
    /**
     * Visitor factory to parse Empty.
     */
    private EmptyFact _emptyfact;

    /**
     * Visitor factory to parse D1a.
     */
    private D1aFact _d1afact;

    /**
     * Constructor for the D1 factory.
     *
     * @param tkz   tokenizer to use
     * @param _emptyfact factory for Empty to use
     * @param _d1afact factory for D1a to use
     */
    public D1Fact(ITokenizer tkz,
                  EmptyFact _emptyfact,
                  D1aFact _d1afact) {
        super(tkz);
        this._emptyfact = _emptyfact;
        this._d1afact = _d1afact;
    }

    /**
     * Constructor for the D1 factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public D1Fact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the Empty factory.
     *
     * @param _emptyfact factory for Empty to use
     */
    public void setEmptyFact(EmptyFact _emptyfact) {
        this._emptyfact = _emptyfact;
    }

    /**
     * Mutator for the D1a factory.
     *
     * @param _d1afact factory for D1a to use
     */
    public void setD1aFact(D1aFact _d1afact) {
        this._d1afact = _d1afact;
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return _emptyfact.makeChainedVisitor(
               _d1afact.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return _emptyfact.makeChainedVisitor(
               _d1afact.makeChainedVisitor(successor));
    }
}
