package output;

/**
 * Factory for E1.
 */
public class E1Fact extends ATVFactory {
    /**
     * Visitor factory to parse Empty.
     */
    private EmptyFact _emptyfact;

    /**
     * Visitor factory to parse E1a.
     */
    private E1aFact _e1afact;

    /**
     * Constructor for the E1 factory.
     *
     * @param tkz   tokenizer to use
     * @param _emptyfact factory for Empty to use
     * @param _e1afact factory for E1a to use
     */
    public E1Fact(ITokenizer tkz,
                  EmptyFact _emptyfact,
                  E1aFact _e1afact) {
        super(tkz);
        this._emptyfact = _emptyfact;
        this._e1afact = _e1afact;
    }

    /**
     * Constructor for the E1 factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public E1Fact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the Empty factory.
     *
     * @param _emptyfact factory for Empty to use
     */
    public void setEmptyFact(EmptyFact _emptyfact) {
        this._emptyfact = _emptyfact;
    }

    /**
     * Mutator for the E1a factory.
     *
     * @param _e1afact factory for E1a to use
     */
    public void setE1aFact(E1aFact _e1afact) {
        this._e1afact = _e1afact;
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return _emptyfact.makeChainedVisitor(
               _e1afact.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return _emptyfact.makeChainedVisitor(
               _e1afact.makeChainedVisitor(successor));
    }
}
