package output;

/**
 * Factory for E.
 */
public class EFact extends ATVFactory {
    /**
     * Visitor factory to parse T.
     */
    private TFact _tfact;

    /**
     * Token visitor to parse T.
     */
    private ITokVisitor _parset;

    /**
     * Visitor factory to parse E1.
     */
    private E1Fact _e1fact;

    /**
     * Token visitor to parse E1.
     */
    private ITokVisitor _parsee1;

    /**
     * Initializer lambda for this factory.
     */
    private ILambda _initializer = new ILambda() {
        public Object apply(Object param) {
            // change state to no-op
            _initializer = NoOpLambda.Singleton;

            // initialize
            _parset = _tfact.makeVisitor();
            _parsee1 = _e1fact.makeVisitor();
            return null;
        }
    };
    /**
     * Constructor for the E factory.
     *
     * @param tkz   tokenizer to use
     * @param _tfact factory for T to use
     * @param _e1fact factory for E1 to use
     */
    public EFact(ITokenizer tkz,
                 TFact _tfact,
                 E1Fact _e1fact) {
        super(tkz);
        this._tfact = _tfact;
        this._e1fact = _e1fact;
    }

    /**
     * Constructor for the E factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public EFact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the T factory.
     *
     * @param _tfact factory for T to use
     */
    public void setTFact(TFact _tfact) {
        this._tfact = _tfact;
    }

    /**
     * Mutator for the E1 factory.
     *
     * @param _e1fact factory for E1 to use
     */
    public void setE1Fact(E1Fact _e1fact) {
        this._e1fact = _e1fact;
    }

    /**
     * Make a E.
     *
     * @param host some token
     * @param param visitor-specific input
     * @return D grammar non-terminal
     */
    private E makeE(AToken host, Object param) {
        return new E((T) host.execute(_parset, param),
                     (E1) nextToken().execute(_parsee1, param));
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        initialize();
        return new ITokVisitor() {
            public Object defaultCase(AToken host, Object param) {
                return new E((T) host.execute(_parset, param),
                             (E1) nextToken().execute(_parsee1, param));
            }
        };
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        initialize();
        return new ITokVisitor() {
            public Object defaultCase(AToken host, Object param) {
                Object o = host.execute(_tfact.makeChainedVisitor(successor), param);
                return (o instanceof T) ? new E((T) o,  (E1) host.execute(_parsee1, param)) : o;
            }
        };
    }

    /**
     * Initialize this factory.
     */
    private void initialize() {
        _initializer.apply(null);
    }
}
