package output;

/**
 * Factory for QuotedStringToken.
 */
public class QuotedStringTokenFact extends ATVFactory {
    /**
     * Constructor for the QuotedStringToken factory.
     *
     * @param tkz   tokenizer to use
     */
    public QuotedStringTokenFact(ITokenizer tkz) {
        super(tkz);
    }


    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return new QuotedStringToken.IQuotedStringTokenVisitor() {
            public Object quotedStringTokenCase(QuotedStringToken host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                throw new IllegalArgumentException("Wrong token: '" + host + "'");
            }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility.
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return new QuotedStringToken.IQuotedStringTokenVisitor() {
            public Object quotedStringTokenCase(QuotedStringToken host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                return host.execute(successor, param);
            }
        };
    }
}
