package output;

/**
 * Factory for T1.
 */
public class T1Fact extends ATVFactory {
    /**
     * Visitor factory to parse WordToken.
     */
    private WordTokenFact _wordtokenfact;

    /**
     * Visitor factory to parse T3.
     */
    private T3Fact _t3fact;

    /**
     * Constructor for the T1 factory.
     *
     * @param tkz   tokenizer to use
     * @param _wordtokenfact factory for WordToken to use
     * @param _t3fact factory for T3 to use
     */
    public T1Fact(ITokenizer tkz,
                  WordTokenFact _wordtokenfact,
                  T3Fact _t3fact) {
        super(tkz);
        this._wordtokenfact = _wordtokenfact;
        this._t3fact = _t3fact;
    }

    /**
     * Constructor for the T1 factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public T1Fact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the WordToken factory.
     *
     * @param _wordtokenfact factory for WordToken to use
     */
    public void setWordTokenFact(WordTokenFact _wordtokenfact) {
        this._wordtokenfact = _wordtokenfact;
    }

    /**
     * Mutator for the T3 factory.
     *
     * @param _t3fact factory for T3 to use
     */
    public void setT3Fact(T3Fact _t3fact) {
        this._t3fact = _t3fact;
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return _wordtokenfact.makeChainedVisitor(
               _t3fact.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return _wordtokenfact.makeChainedVisitor(
               _t3fact.makeChainedVisitor(successor));
    }
}
