package output;

/**
 * Factory for T2.
 */
public class T2Fact extends ATVFactory {
    /**
     * Constructor for the T2 factory.
     *
     * @param tkz   tokenizer to use
     */
    public T2Fact(ITokenizer tkz) {
        super(tkz);
    }


    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return new T2.IT2Visitor() {
            public Object t2Case(T2 host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                throw new IllegalArgumentException("Wrong token: '" + host + "'");
            }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility.
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return new T2.IT2Visitor() {
            public Object t2Case(T2 host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                return host.execute(successor, param);
            }
        };
    }
}
