package output;

/**
 * Factory for T3.
 */
public class T3Fact extends ATVFactory {
    /**
     * Visitor factory to parse Empty.
     */
    private EmptyFact _emptyfact;

    /**
     * Visitor factory to parse T.
     */
    private TFact _tfact;

    /**
     * Constructor for the T3 factory.
     *
     * @param tkz   tokenizer to use
     * @param _emptyfact factory for Empty to use
     * @param _tfact factory for T to use
     */
    public T3Fact(ITokenizer tkz,
                  EmptyFact _emptyfact,
                  TFact _tfact) {
        super(tkz);
        this._emptyfact = _emptyfact;
        this._tfact = _tfact;
    }

    /**
     * Constructor for the T3 factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public T3Fact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the Empty factory.
     *
     * @param _emptyfact factory for Empty to use
     */
    public void setEmptyFact(EmptyFact _emptyfact) {
        this._emptyfact = _emptyfact;
    }

    /**
     * Mutator for the T factory.
     *
     * @param _tfact factory for T to use
     */
    public void setTFact(TFact _tfact) {
        this._tfact = _tfact;
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return _emptyfact.makeChainedVisitor(
               _tfact.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return _emptyfact.makeChainedVisitor(
               _tfact.makeChainedVisitor(successor));
    }
}
