package output;

/**
 * Factory for T.
 */
public class TFact extends ATVFactory {
    /**
     * Visitor factory to parse T1.
     */
    private T1Fact _t1fact;

    /**
     * Visitor factory to parse T2.
     */
    private T2Fact _t2fact;

    /**
     * Constructor for the T factory.
     *
     * @param tkz   tokenizer to use
     * @param _t1fact factory for T1 to use
     * @param _t2fact factory for T2 to use
     */
    public TFact(ITokenizer tkz,
                 T1Fact _t1fact,
                 T2Fact _t2fact) {
        super(tkz);
        this._t1fact = _t1fact;
        this._t2fact = _t2fact;
    }

    /**
     * Constructor for the T factory.
     * This does not set the factories, they have to be set using the setXXXFact() methods.
     *
     * @param tkz   tokenizer to use
     */
    public TFact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Mutator for the T1 factory.
     *
     * @param _t1fact factory for T1 to use
     */
    public void setT1Fact(T1Fact _t1fact) {
        this._t1fact = _t1fact;
    }

    /**
     * Mutator for the T2 factory.
     *
     * @param _t2fact factory for T2 to use
     */
    public void setT2Fact(T2Fact _t2fact) {
        this._t2fact = _t2fact;
    }

    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return _t1fact.makeChainedVisitor(
               _t2fact.makeVisitor());
    }

    /**
     * Make a token visitor to parse an E non-terminal that decorates the given visitor.
     *
     * @param successor visitor to decorate
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return _t1fact.makeChainedVisitor(
               _t2fact.makeChainedVisitor(successor));
    }
}
