package output;

import java.io.*;

/**
 * Use the StreamTokenizer provided by in java.io to scan an input stream and extract an appropriate AToken.
 */
public class Tokenizer implements ITokenizer {
    /**
     * StreamTokenizer to use.
     */
    private StreamTokenizer _st;

    /**
     * Reader to use.
     */
    private Reader _fileReader;

    /**
     * Initialize _st to read from a input Reader file with the given input file name.
     *
     * @param inputFileName the name of the input text file
     * @throws java.io.FileNotFoundException
     */
    public Tokenizer(String inputFileName) throws FileNotFoundException {
        // Create an input text file object:
        _fileReader = new FileReader(inputFileName);

        // Create a Streamtokenizer to parse the input text file:
        _st = new StreamTokenizer(_fileReader);

        // TODO: set up StreamTokenizer
    }

    /**
     * Use _st to scan the input, extracts, and returns an appropriate concrete AToken.
     *
     * @return next token
     * @throws IllegalArgumentException Thrown if an illegal input is encountered.
     */
    public AToken getNextToken() {
        try {
            if (StreamTokenizer.TT_EOF != _st.nextToken()) {
                switch (_st.ttype) {
                    // TODO: cases for Tokens
                    // case '?':
                    //    {
                    //        return ...;
                    //    }
                    // case StreamTokenizer.TT_WORD:
                    //     {
                    //         return new WordToken(_st.sval);
                    //     }
                    default:
                        throw new IllegalArgumentException("Illegal token (ttype=" + new Character((char)_st.ttype) + ", sval=" + _st.sval + ", nval=" + _st.nval + ")!");
                }
            }
            else {
                _fileReader.close();
                return EOFToken.Singleton;
            }
        }
        catch (IOException e) {
            return null;
        }
    }

    /**
     * Put the previously consumed token back into the token stream.
     * Can only put back one token.
     */
    public void putBack() {
        _st.pushBack();
    }
}
