package output;

/**
 * Factory for WordToken.
 */
public class WordTokenFact extends ATVFactory {
    /**
     * Constructor for the WordToken factory.
     *
     * @param tkz   tokenizer to use
     */
    public WordTokenFact(ITokenizer tkz) {
        super(tkz);
    }


    /**
     * Make a token visitor to parse an E non-terminal.
     *
     * @return token visitor
     */
    public ITokVisitor makeVisitor() {
        return new WordToken.IWordTokenVisitor() {
            public Object wordTokenCase(WordToken host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                throw new IllegalArgumentException("Wrong token: '" + host + "'");
            }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility.
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ITokVisitor makeChainedVisitor(final ITokVisitor successor) {
        return new WordToken.IWordTokenVisitor() {
            public Object wordTokenCase(WordToken host, Object param) {
                return host;
            }

            public Object defaultCase(AToken host, Object param) {
                return host.execute(successor, param);
            }
        };
    }
}
